#include <sys/types.h>
#include <sys/stat.h>
#include <pwd.h>
#include <utmp.h>

#include <unistd.h>
#include <fcntl.h>

#include <vector>
#include <string>
#include <stdexcept>

#include "user.h"

namespace machine {
    void read_users(const string& file, vector<user>& users) 
	throw(runtime_error) {
	string utmp = _PATH_UTMP;
	if ( file.size() ) utmp = file;
	int f = open(utmp.c_str(), O_RDONLY);
	int r;
	struct utmp ut;


	if ( f == -1 ) throw runtime_error("Can't open " + utmp);
	while ( ( r = read(f,&ut, sizeof(ut))) > 0 ) {
	    if ( ut.ut_name[0] != '\0' ) {
		user u;
		struct stat st;
		struct passwd *pwd;
		string dev = "/dev/";

		u.uname().append(ut.ut_name, UT_NAMESIZE);
		u.tty().append(ut.ut_line, UT_LINESIZE);
		u.host().append(ut.ut_host, UT_HOSTSIZE);
		u.when() = ut.ut_time;
		dev += u.tty(); 
		if ( stat(dev.c_str(), &st) != -1 ) 
		    u.idle() = now() - st.st_mtimespec.tv_sec;
		
		if ( (pwd = getpwnam(u.uname().c_str()))) {
		    string n = pwd->pw_gecos;

		    if ( n.find(',') != string::npos) 
			n.erase(n.find(','));
		    u.name() = n;
		}
		users.push_back(u);
	    }
	}
	if ( r != 0 ) throw runtime_error("Error reading " + utmp);
	close(f);
    }
};
