#include <sys/types.h>
#include <sys/stat.h>
#include <pwd.h>
#include <utmp.h>

#include <unistd.h>
#include <fcntl.h>

#include <vector>
#include <string>
#include <stdexcept>

#include "user.h"

namespace machine {
    void read_users(const string& file, vector<user>& users) 
	    throw(runtime_error) {
	struct utmp *ut;

	if ( file.size()) utmpname(file.c_str());
	setutent();
	while ( ( ut = getutent()) ) {
	    if ( ut->ut_type != USER_PROCESS ) continue;
	    else {
		user u;
		struct stat st;
		struct passwd *pwd;
		string dev = "/dev/";

		u.uname().append(ut->ut_name, UT_NAMESIZE);
		u.tty().append(ut->ut_line, UT_LINESIZE);
		u.host().append(ut->ut_host, UT_HOSTSIZE);
		u.when() = ut->ut_time;
		dev += u.tty();
		if ( stat(dev.c_str(), &st) != -1 ) 
		    u.idle() = time(0) - st.st_mtime;
		if ( (pwd = getpwnam(u.uname().c_str()))) {
		    string n = pwd->pw_gecos;

		    if ( n.find(',') != string::npos) 
			n.erase(n.find(','));
		    u.name() = n;
		}
		users.push_back(u);
	    }
	}
	endutent();
    }
};
