#include <sys/types.h>
#include <utmp.h>

#include <sys/errno.h>
#include <unistd.h>

#include <fcntl.h>

#include <iostream>
#include <vector>
#include <string>

#include <algorithm>

#include "config.h"

#include "user.h"

string version=PACKAGE_STRING" by Ted Faber\nCompiled on "OS_VERSION;

// failure routines.  Print the message and any errno message to the standard
// error and bail out.  There's an adapter to call fatal with strings directly.
inline void fatal(const char *message=0) {
    if ( message && message[0] != '\0' ) cerr << message << endl;
    if (errno) cerr << strerror(errno) << endl;
    exit(20);
}

inline void fatal(const string& message) {
    fatal(message.c_str());    
}

inline void warn(const string& message) {
    cerr << message << endl;
}

namespace machine { void read_users(const string&, vector<user>&); }

class print_user {
    protected:
	ostream& f;
	bool name, host, idle, line, intime, uname, secs;
    public:
	print_user(ostream &st, bool n=true, bool h=false, bool i=false, 
		bool l=false, bool it=false, bool u=false, bool se=false) : 
	    f(st), name(n), host(h), idle(i), line(l), 
	    intime(it), uname(u), secs(se) { }
	void operator()(const user &u) const {
	    if ( name ) f << u.name();
	    if ( uname )  {
		if ( name ) f << " (" << u.uname() << ")";
		else f << u.uname();
	    }
	    if ( line ) f << " logged on " << u.tty();
	    if ( host ) f << " from " << u.host();
	    if ( intime ) f << " at " << u.when();
	    if ( idle && u.idle().t_val() )  {
		f << " idle for " ;
		if ( secs ) f << u.idle().t_val() << " secs";
		else f << u.idle();
	    }
	    f << endl;
	}
};

int main(int argc, char **argv) {
    vector<user> users;
    bool name = true;
    bool host = false;
    bool idle = false;
    bool line = false;
    bool intime = false;
    bool uname = false;
    bool secs = false;
    const char *opts = "NnHhIiLlTtUuSsAaf:V";
    string file;
    int c;

    while (( c = getopt(argc, argv, opts)) != -1 ) {
	switch (c) {
	    case 'N':
	    case 'n':
		name = (c == 'n');
		break;
	    case 'H':
	    case 'h':
		host = (c == 'h');
		break;
	    case 'I':
	    case 'i':
		idle = (c == 'i');
		break;
	    case 'L':
	    case 'l':
		line = (c == 'l');
		break;
	    case 'T':
	    case 't':
		intime = (c == 't');
		break;
	    case 'U':
	    case 'u':
		uname = (c == 'u');
		break;
	    case 'S':
	    case 's':
		idle = true;
		secs = (c == 's');
		break;
	    case 'f':
		file = optarg;
		break;
	    case 'a':
	    case 'A':
		name = host = idle = line = intime = uname = (c == 'a');
		break;
	    case 'V':
		cout << version << endl;
		exit(0);
		break;
	}
    }
    if ( !(name || host || idle || line || intime || uname ) ) {
	warn("No fields selected, using name (-n)");
	name = true;
    }

    machine::read_users(file, users);
    for_each(users.begin(), users.end(), 
	    print_user(cout, name, host, idle, line, intime, uname, secs));
}

