#ifndef USER_H
#define USER_H

#include <iostream>
#include <iomanip>

#include "Time.h"

// Make an idle_time print in the idle_time format
class idle_time : public Time {
    public:
	idle_time(time_t t=0): Time(t) { }
	idle_time(const struct timeval& t) : Time(t) { }
	idle_time(double d) : Time(d) { }
	idle_time(const Time &t) : Time(t) { }
	ostream& print(ostream& f) const {
	    int secs = t_val();
	    int days = secs / (24*60 *60);
	    secs %= 24 * 60 * 60;
	    int hours = secs / 3600;
	    secs %= 3600;
	    int mins = secs/ 60;
	    secs %= 60;

	    ios::fmtflags fl = f.flags();
	    char fill = f.fill();

	    if ( days ) 
		f << days << " day" << ((days > 1) ? "s " : " ");

	    f.setf(ios::fixed);
	    f.fill('0');
	    f << setw(2) << hours << ":" ;
	    f << setw(2) << mins << ":" ;
	    f << setw(2) << secs ;
	    f.fill(fill);
	    f.flags(fl);
	    return f;
	}
};

// Make a login time print as a login time.
class login_time : public Time {
    public:
	login_time(time_t t=0): Time(t) { }
	login_time(const struct timeval& t) : Time(t) { }
	login_time(double d) : Time(d) { }
	login_time(const Time &t) : Time(t) { }
	ostream& print(ostream& f) const { 
	    time_t now = time(0);

	    if ( now - t_val() < 24 * 60 * 60 ) {
		// The login_time is less than a day ago
		struct tm t = tm_val();

		ios::fmtflags fl = f.flags();
		char fill = f.fill();

		f.setf(ios::fixed);
		f.fill('0');
		f << setw(2) << t.tm_hour << ":" ;
		f << setw(2) << t.tm_min << ":" ;
		f << setw(2) << t.tm_sec ;
		f.fill(fill);
		f.flags(fl);
		return f;
	    }
	    else {
		time_t t = t_val();
		char *ts = ctime(&t);
		ts[24] = '\0';
		return f << ts;
	    }
	}
};

class user {
    protected:
	string un;
	string n;
	string t;
	string h;
	login_time w;
	idle_time i;
    public:
	user() : un(), n(), t(), h(), w(), i() { }
	const string uname() const { return un; }
	string& uname() { return un; }
	const string name() const { return n; }
	string& name() { return n; }
	const string tty() const { return t; }
	string& tty() { return t; }
	const string host() const { return h; }
	string& host() { return h; }
	const login_time when() const { return w; }
	login_time& when() { return w; }
	const idle_time idle() const { return i; }
	idle_time& idle() { return i; }
	ostream& print(ostream& f) const { 
	    f << n << " (" << un << ") " << t;
	    f << " " << h << " " << w << " " << i; 
	    return f;
	}
};

inline ostream& operator<<(ostream& f, const user& u) { return u.print(f); }
#endif
